/*
  c8zy_test.c

  2025-02-25 - 2025-03-09
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "collect.h"
#include "craft.h"
#include "items.h"
#include "shapes.h"
#include "tools.h"
#include "player.h"
#include "map.h"
#include "io.h"


int main(int argc, char * argv[]) {
  int pyy;
  unsigned char i, done, found, cmd;
  char msg[41];

  io_init();

  /* Initialize player state */
  player_init();

  /* Initialize map object states */
  map_init();

  /* --- MAIN LOOP! --- */
  done = 0;
  while (!done) {
    /* --- Display --- */
    io_draw_stats();
    io_draw_inventory(NULL);
    io_draw_map();

    /* --- Handle command input --- */
    cmd = io_get_cmd();

    pyy = py * W;

    /* MOVEMENT */
    if (cmd == CMD_DIR_UP) {
      if (py > 0 && walkable(map[pyy - W + px])) {
        player_move(DIR_UP);
      }
    } else if (cmd == CMD_DIR_LEFT) {
      if (px > 0 && walkable(map[pyy + px - 1])) {
        player_move(DIR_LEFT);
      }
    } else if (cmd == CMD_DIR_DOWN) {
      if (py < H - 1 && walkable(map[pyy + W + px])) {
        player_move(DIR_DOWN);
      }
    } else if (cmd == CMD_DIR_RIGHT) {
      if (px < W - 1 && walkable(map[pyy + px + 1])) {
        player_move(DIR_RIGHT);
      }
    } else if (cmd == CMD_QUIT) {
      /* QUIT */
      done = 1;
    } else if (cmd == CMD_GATHER) {
      /* GATHER */
      int wx, wy, res;

      io_get_dir("Gather", &wx, &wy);

      res = gather(wx, wy);

      if (res == GATHER_FULL) {
        sprintf(msg, "FULL! You can't carry any more!");
      } else if (res == GATHER_OOB) {
        sprintf(msg, "OUT OF BOUNDS!");
      } else if (res == GATHER_BAD) {
        sprintf(msg, "Sorry, you can't gather that!");
      } else {
        sprintf(msg, "You got %s!", item_names[res]);
      }
      io_show_msg(msg);
    } else if (cmd == CMD_PLACE) {
      /* PLACE/DROP */
      int item_idx;

      item_idx = io_get_item_idx("Place");

      if (item_idx != ITEM_BAD && items[item_idx] != ITEM_NONE) {
        int wx, wy;

        sprintf(msg, "Drop %s where?", item_names[items[item_idx]]);
        io_get_dir(msg, &wx, &wy);

        if (wx < 0 || wy < 0 || wx >= W || wy >= H) {
          /* OUT OF BOUNDS! */
        } else {
          int res;

          sprintf(msg, "You dropped %s", item_names[items[item_idx]]);

          res = place(item_idx, wx, wy);
          if (res == PLACE_DROP) {
            /* Set the msg above, b/c the item will be gone at this point! */
          } else if (res == PLACE_BAD) {
            sprintf(msg, "%s doesn't mix with that", item_names[items[item_idx]]);
          } else {
            sprintf(msg, "You crafted %s!", item_names[res]);
          }

          io_show_msg(msg);
        }
    }
    } else if (cmd == CMD_USE) {
      /* USE */
      int item_idx;

      item_idx = io_get_item_idx("Use");

      if (item_idx != ITEM_BAD && items[item_idx] != ITEM_NONE) {
        int wx, wy, res;

        sprintf(msg, "Use %s where?", item_names[items[item_idx]]);
        io_get_dir(msg, &wx, &wy);

        if (wx < 0 || wy < 0 || wx >= W || wy >= H) {
          /* OUT OF BOUNDS! */
        } else {
          res = use_tool(item_idx, wx, wy);

          if (res == USE_TOOL_FULL) {
            sprintf(msg, "You can't carry any more!");
          } else if (res == USE_TOOL_BAD) {
            sprintf(msg, "You cannot use %s on that", item_names[items[item_idx]]);
          } else if (res == USE_TOOL_NONE) {
            sprintf(msg, "You used your %s", item_names[items[item_idx]]);
          } else {
            sprintf(msg, "You created %s!", item_names[res]);
          }
          io_show_msg(msg);
        }
      }
    } else if (cmd == CMD_EAT) {
      /* EAT */
      int item_idx;

      item_idx = io_get_item_idx("Eat");

      if (item_idx != ITEM_NONE && item_idx != ITEM_BAD) {
        int res;

        sprintf(msg, "You ate %s", item_names[items[item_idx]]);

        res = player_eat(item_idx);

        if (res == EAT_BAD) {
          sprintf(msg, "You can't eat %s!", item_names[items[item_idx]]);
        } else {
          /* Set the msg above, b/c the item will be gone at this point! */
        }

        io_show_msg(msg);
      }
    } else if (cmd == CMD_CRAFT) {
      /* CRAFT */
      int craft_done, item_idx, num_craft_items;
      unsigned char craft_item_idx[CRAFT_ITEM_CT];

      num_craft_items = 0;
      for (i = 0; i < CRAFT_ITEM_CT; i++) {
        craft_item_idx[i] = MAX_ITEMS;
      }

      craft_done = 0;

      /* Collect list of items to craft */
      while (craft_done == 0) {
        io_draw_inventory(craft_item_idx);

        item_idx = io_get_craft_item_idx("Craft");

        if (item_idx == ITEM_STOP) {
          craft_done = 1;
        } else if (item_idx != ITEM_BAD && items[item_idx] != ITEM_NONE) {
          /* Valid slot, but see if we already added it */
          found = 0;
          for (i = 0; i < num_craft_items && found == 0; i++) {
            if (craft_item_idx[i] == item_idx) {
              found = 1;
            }
          }
          if (!found) {
            /* Okay to add this item! */
            craft_item_idx[num_craft_items] = item_idx;
            num_craft_items++;

            if (num_craft_items == CRAFT_ITEM_CT) {
              /* Stop automatically once we hit four items */
              craft_done = 1;
            }
          }
        }
      }

      if (num_craft_items < 2) {
        /* Must have 2 or more items to craft! */
        sprintf(msg, "Need to combine at least two items!");
      } else {
        int res;

        res = craft(craft_item_idx, num_craft_items);

        if (res == CRAFT_BAD) {
          sprintf(msg, "That combo doesn't make anything");
        } else {
          sprintf(msg, "You crafted %s!", item_names[res]);
        }
      }

      io_show_msg(msg);
    }

    /* FIXME: Increase thirst */
    /* FIXME: Increase hunger */

    // FIXME map_process();
  }

  return(0);
}
