/*
  collect.c

  For C8zy.  Mapping of what is collected when items are
  gathered from the map, and what is placed onto the map.
  Bill Kendrick <bill@newbreedsoftware.com>

  2025-03-01 - 2025-03-02
*/

#include "collect.h"
#include "player.h"
#include "map.h"
#include "items.h"
#include "shapes.h"

/* For each map shape, three elements:
 * 0. the shape being collect off the map
 * 1. the shape being left in its place
 * 2. what, if anything, is collected into inventory
 */
unsigned char collect[][3] = {
  {SH_DIRT,       SH_MOUND,      ITEM_DIRT},
  {SH_SAND,       SH_SAND,       ITEM_SAND},
  {SH_WATER_SHAL, SH_WATER_SHAL, ITEM_WATER},
  {SH_WATER_DEEP, SH_WATER_DEEP, ITEM_WATER},
  {SH_GRASS,      SH_DIRT,       ITEM_GRASS},
  {SH_MOUND,      SH_DIRT,       ITEM_DIRT},
  {SH_SPROUT,     SH_MOUND,      ITEM_NONE},
  {SH_FLOWER1_A,  SH_DIRT,       ITEM_FLOWER1_A},
  {SH_FLOWER1_B,  SH_DIRT,       ITEM_FLOWER1_B},
  {SH_FLOWER2_A,  SH_DIRT,       ITEM_FLOWER2_A},
  {SH_FLOWER2_B,  SH_DIRT,       ITEM_FLOWER2_B},
  {SH_MELON_A,    SH_DIRT,       ITEM_MELON_A},
  {SH_MELON_B,    SH_DIRT,       ITEM_MELON_B},
  {SH_SEEDS_A,    SH_DIRT,       ITEM_SEEDS_A /* FIXME: what kind? */ },
  {SH_SEEDS_B,    SH_DIRT,       ITEM_SEEDS_B /* FIXME: what kind? */ },
  {SH_LOG,        SH_DIRT,       ITEM_LOG},
  {SH_PLANK,      SH_DIRT,       ITEM_PLANK},
  {SH_GEM_1,      SH_DIRT,       ITEM_GEM_1},
  {SH_GEM_2,      SH_DIRT,       ITEM_GEM_2},
  {SH_GEM_3,      SH_DIRT,       ITEM_GEM_3},
  {SH_GEM_4,      SH_DIRT,       ITEM_GEM_4},
  {SH_GEM_5,      SH_DIRT,       ITEM_GEM_5},
  {SH_ROCK_SM,    SH_DIRT,       ITEM_ROCK},
  {SH_TREE,       SH_TREE_BARE,  ITEM_LEAVES},
  {SH_TREE_FRUIT, SH_TREE,       ITEM_FRUIT},
  {SH_LEAVES,     SH_DIRT,       ITEM_LEAVES},
  {SH_CRAB,       SH_DIRT /* FIXME */, ITEM_CRAB},
  {SH_CRAB_COOK,  SH_DIRT,       ITEM_CRAB_COOK},
  {SH_FISH,       SH_WATER_SHAL /* FIXME */, ITEM_FISH},
  {SH_FISH_COOK,  SH_DIRT,       ITEM_FISH_COOK},
  {SH_FISH_BONE,  SH_DIRT,       ITEM_FISH_BONE},
  {SH_SNAIL,      SH_DIRT /* FIXME */, ITEM_SNAIL},
  {SH_SHELL,      SH_DIRT /* FIXME */, ITEM_SHELL},
  {SH_BREAD,      SH_DIRT,       ITEM_BREAD},

  {SH_YOU,        SH_YOU,        ITEM_NONE}, /* marks end of the list */
};


int gather(int wx, int wy) {
  int found, i, free_slot;
  int wyy;
  unsigned char ch;

  if (wx < 0 || wy < 0 || wx >= W || wy >= H) {
    return GATHER_OOB;
  }

  wyy = wy * W;

  free_slot = get_free_item_slot();
  if (free_slot == MAX_ITEMS) {
    return GATHER_FULL;
  }

  ch = map[wyy + wx];

  found = -1;
  for (i = 0; collect[i][0] != SH_YOU && found == -1; i++) {
    if (collect[i][0] == ch) {
      found = i;
    }
  }

  if (found == -1) {
    return(GATHER_BAD);
  }

  map[wyy + wx] = collect[found][1];
  items[free_slot] = collect[found][2];
  tired = tired + TIRED_GATHER;

  return(collect[found][2]);
}
