/*
  map.c

  For C8zy.  Map-related stuff.
  Bill Kendrick <bill@newbreedsoftware.com>

  2025-03-01 - 2025-03-09
*/

#include <stdlib.h>
#include "map.h"
#include "shapes.h"

unsigned char map[H * W] = {
  SH_WATER_DEEP, SH_WATER_DEEP, SH_WATER_SHAL, SH_SAND,    SH_MOUND, SH_MOUND, SH_DIRT,  SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_TREE,
  SH_WATER_DEEP, SH_WATER_SHAL, SH_WATER_SHAL, SH_SAND,    SH_SNAIL, SH_MOUND, SH_DIRT,  SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_TREE,
  SH_WATER_SHAL, SH_WATER_SHAL, SH_SAND,       SH_SAND,    SH_MOUND, SH_MOUND, SH_DIRT,  SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_TREE,
  SH_SAND,       SH_SAND,       SH_SAND,       SH_DIRT,    SH_DIRT,  SH_DIRT,  SH_DIRT,  SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_TREE,
  SH_SAND,       SH_DIRT,       SH_DIRT,       SH_DIRT,    SH_DIRT,  SH_GRASS, SH_GRASS, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_TREE,
  SH_DIRT,       SH_FIRE,       SH_DIRT,       SH_DIRT,    SH_DIRT,  SH_GRASS, SH_GRASS, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_TREE,
  SH_LOG,        SH_DIRT,       SH_FLOWER2_A,  SH_DIRT,    SH_DIRT,  SH_GRASS, SH_GRASS, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_TREE,
  SH_TREE,       SH_SEEDS_A,    SH_DIRT,       SH_ROCK_SM, SH_DIRT,  SH_DIRT,  SH_DIRT,  SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_DIRT, SH_TREE,
};

unsigned char map_meta[H][W];
unsigned char map_under[H][W];

/* FIXME: combine with PLAYER_DIR in? */
signed char dir_x[4] = { 0, 1, 0, -1 };
signed char dir_y[4] = { -1, 0, 1, 0 };

void map_init(void) {
  unsigned char x, y;

  /* FIXME: Not used for anything yet! */
  for (y = 0; y < H; y++) {
    for (x = 0; x < W; x++) {
      map_meta[y][x] = 0;
      map_under[y][x] = 0;
    }
  }
}

void map_process(void) {
  unsigned char i;
  unsigned char is_moving_shape, d;
  int x, y, nx, ny, yy, nyy;
  unsigned char cur_map_ch, new_map_ch;

  /* Cellular automata procedure against the map */
  for (y = 0; y < H; y++) {
    yy = y * W;

    for (x = 0; x < W; x++) {
      cur_map_ch = map[yy + x];

      /* Pick a random direction & try to spread there */
      d = rand() % 4;
      nx = x + dir_x[d];
      ny = y + dir_y[d];
      nyy = ny * W;

      if (nx > 0 && nx < W - 1 && ny > 0 && ny < H - 1) {
        new_map_ch = map[nyy + nx];
  
        for (i = 0; spreading_shapes[i] != SH_YOU; i = i + 3) {
          if (spreading_shapes[i] == cur_map_ch &&
              spreading_shapes[i + 1] == new_map_ch) {
            map[nyy + nx] = spreading_shapes[i + 2];
            break;
          }
        }
      }

      /* Pick a random chance of growing FIXME */
      d = rand() % 12;
      if (d == 0) {
        for (i = 0; growing_shapes[i][0] != SH_YOU; i++) {
          if (growing_shapes[i][0] == cur_map_ch) {
            map[yy + x] = growing_shapes[i][1];
            break;
          }
        }
      }

      /* Move */
      is_moving_shape = 0;
      for (i = 0; moving_shapes[i][0] != SH_YOU && is_moving_shape == 0; i++) {
        if (moving_shapes[i][0] == cur_map_ch) {
          is_moving_shape = 1;
        }
      }

#if 0
      /* FIXME: This doesn't work yet */
      if (is_moving_shape) {
        unsigned char m, t;

        m = map_meta[y][x];

        printch(map[y][x]);
        printf(" at (%2d,%2d) is a moving shape!  Meta = %08b\n", x, y, m);

        d = (m & 0b01100000) >> 5;
        t = (m & 0b00011111);
        t--;
        m = ((d << 5) | t);
        map_meta[y][x] = m;

        if (t == 0) {
          /* Timer ran out; pick a new direction */
          d = rand() % 4;
          t = (rand() & 8) + 8;

          printf("New dir %d for %d steps", d, t);
          map_meta[y][x] = ((d << 5) | t);
        } else if (t % 2 == 0) {
          /* Time to move */
          d = (m >> 5);
          nx = x + dir_x[d];
          ny = y + dir_y[d];

          if (nx >= 0 && nx < W && ny >= 0 && ny < H) {
            printf("Trying to move %d (from %2d,%2d to %2d, %2d)!\n", d, x, y, nx, ny);

            applied = 0;
            for (i = 0; moving_shapes[i][0] != SH_YOU && !applied; i++) {
              if (map[ny][nx] == moving_shapes[i][1]) {
                /* Copy our metadata over to the new spot */
                map_meta[ny][nx] = map_meta[y][x];
                /* Clear metadata at the old spot */
                map_meta[y][x] = 0;

                /* Reset map shape to what was there before */
                map[y][x] = map_under[y][x];
                /* Clear what was there before */
                map_under[y][x] = 0;

                /* Capture what shape was at the map at the new spot */
                map_under[ny][nx] = map[ny][nx];
                /* Place the new shape in the new spot */
                map[ny][nx] = moving_shapes[i][3];

                applied = 1;
              }
            }
            if (applied) {
              printf("Did move!\n");
              if (nx == x + 1) {
                /* Next loop iteration, skip ourselves if we moved right! */
                x++;
              }
            }
          }
        }
      }
#endif

    }
  }
}
