/*
  player.c

  For C8zy.  Player data and logic.
  Bill Kendrick <bill@newbreedsoftware.com>

  2025-03-01 - 2025-03-02
*/

#include "player.h"
#include "shapes.h"
#include "items.h"
#include "map.h"

int px, py;
int hunger, thirst, tired;

unsigned char items[MAX_ITEMS];

void player_init(void) {
  int i;

  px = W / 2;
  py = H / 2;
  hunger = 24;
  thirst = 32;
  tired = 12;

  for (i = 0; i < MAX_ITEMS; i++) {
    items[i] = ITEM_NONE;
  } 
  items[0] = ITEM_AX; // FIXME
  items[1] = ITEM_DOUGH; // FIXME
}

int walkable(unsigned char ch) {
  int i;

  for (i = 0; walkable_shapes[i] != SH_YOU; i++) {
    if (ch == walkable_shapes[i]) {
      return 1;
    }
  }

  return 0;
}

int get_free_item_slot(void) {
  int i, free_slot;

  free_slot = MAX_ITEMS;

  for (i = 0; i < MAX_ITEMS && free_slot == MAX_ITEMS; i++) {
    if (items[i] == ITEM_NONE) {
      free_slot = i;
    }
   }

  return free_slot;
}

void player_move(unsigned char dir) {
  int pyy;

  pyy = py * W;

  if (dir == DIR_UP && py > 0 && walkable(map[pyy - W + px])) {
    py--;
    tired = tired + TIRED_MOVE;
  } else if (dir == DIR_RIGHT && px < W - 1 && walkable(map[pyy + px + 1])) {
    px++;
    tired = tired + TIRED_MOVE;
  } else if (dir == DIR_DOWN && py < H - 1 && walkable(map[pyy + 1 + px])) {
    py++;
    tired = tired + TIRED_MOVE;
  } else if (dir == DIR_LEFT && px > 0 && walkable(map[pyy + px - 1])) {
    px--;
    tired = tired + TIRED_MOVE;
  }
}

int player_eat(int item) {
  int i, found;

  found = -1;
  for (i = 0; food[i][0] != ITEM_NONE && found == -1; i++) {
    if (food[i][0] == items[item]) {
      found = i;
    }
  }
  if (found == -1) {
    return(EAT_BAD);
  }

  hunger = hunger - (food[found][1] - 127);
  if (hunger < 0) {
    hunger = 0;
  }
  /* FIXME: Prevent overflow */
 
  thirst = thirst - (food[found][2] - 127);
  if (thirst < 0) {
    thirst = 0;
  }
  /* FIXME: Prevent overflow */
 
  items[item] = ITEM_NONE;

  return(EAT_GOOD);
}

