/*
  shapes.c

  For C8zy.  Constants that define each shape in the game.
  Bill Kendrick <bill@newbreedsoftware.com>

  2025-03-01 - 2025-03-01
*/

#include "shapes.h"


unsigned char walkable_shapes[] = {
  SH_DIRT,
  SH_SAND,
  SH_WATER_SHAL,
  SH_GRASS,
  SH_MOUND,
  SH_SPROUT,
  SH_FLOWER1_A,
  SH_FLOWER1_B,
  SH_FLOWER2_A,
  SH_FLOWER2_B,
  SH_SEEDS_A,
  SH_SEEDS_B,
  SH_PLANK,
  SH_DOOR,
  SH_BRIDGE,
  SH_LEAVES,

  SH_YOU, /* marks end of list */
};


/* Some shapes can spread */
/* For each pair of:
 *   0. the shape on the map, if it wants to spread onto...  (SH_...)
 *   1. ...the shape in an adjacent position  (SH...)
 * we have:
 *   2. what shape results...       (SH)...)
 */
unsigned spreading_shapes[] = {
  SH_FIRE, SH_LOG,    SH_FIRE,
  SH_FIRE, SH_FENCE,  SH_FIRE,
  SH_FIRE, SH_BRIDGE, SH_WATER_SHAL /* FIXME */,
  SH_FIRE, SH_TREE,   SH_TREE_BURNT,

  SH_FLOWER1_A, SH_MOUND, SH_SPROUT /* FIXME */,
  SH_FLOWER1_B, SH_MOUND, SH_SPROUT /* FIXME */,
  SH_FLOWER2_A, SH_MOUND, SH_SPROUT /* FIXME */,
  SH_FLOWER2_B, SH_MOUND, SH_SPROUT /* FIXME */,

  SH_YOU,       SH_YOU,   SH_YOU, /* marks end of list */
};

/* Some shapes grow into other shapes */
/* For each shape (SH_...), the shape it grows into (SH_...) */
/* FIXME: "Chance" value for each! */
unsigned growing_shapes[][2] = {
//  { SH_SPROUT,    SH_FLOWER1_A /* FIXME */ },
  { SH_TREE_BARE, SH_TREE },
  { SH_TREE,      SH_TREE_FRUIT },

  { SH_YOU,       SH_YOU }, /* marks end of list */
};

/* Moving shapes (creatures) */
/* For each:
 *   0. creature that moves (SH_...), and...
 *   1. ...thing it will move onto (SH_...),
 * we have:
 *   2. ...how it affects the thing
 */
unsigned moving_shapes[][3] = {
  { SH_SNAIL,     SH_DIRT,       SH_DIRT  },
  { SH_SNAIL,     SH_MOUND,      SH_MOUND },
  { SH_SNAIL,     SH_SPROUT,     SH_MOUND },
  { SH_SNAIL,     SH_FLOWER1_A,  SH_MOUND },
  { SH_SNAIL,     SH_FLOWER1_B,  SH_MOUND },
  { SH_SNAIL,     SH_FLOWER2_A,  SH_MOUND },
  { SH_SNAIL,     SH_FLOWER2_B,  SH_MOUND },
  { SH_SNAIL,     SH_FISH_COOK,  SH_FISH_BONE },
  { SH_SNAIL,     SH_CRAB_COOK,  SH_DIRT },
  { SH_SNAIL,     SH_GRASS,      SH_GRASS },

  { SH_YOU,       SH_YOU,        SH_YOU },  /* marks end of list */
};
