/*
  tools.c

  For C8zy.  Mapping of what is items can act as tools, and
  how they affect shapes on the map.
  Bill Kendrick <bill@newbreedsoftware.com>

  2025-03-01 - 2025-03-02
*/

#include "tools.h"
#include "shapes.h"
#include "items.h"
#include "map.h"
#include "player.h"

/* FIXME: We could include tool damage amounts */

/* For each pair of:
 *   0. the item from inventory, being combined with...  (ITEM_...)
 *   1. ...the shape on the map                          (SH...)
 * we have:
 *   2. what shape remains on the map                    (SH_...)
 *   3. how much effort it takes (tired increase amount)
 */
unsigned char tools_onland[][4] = {
  { ITEM_AX,     SH_TREE,      SH_LOG,    12 }, /* ax chops tree into log */
  { ITEM_AX,     SH_LOG,       SH_PLANK,   8 }, /* ax chops log into plank */
  { ITEM_HAMMER, SH_ROCK_LG,   SH_ROCK_SM, 6 }, /* hammer breaks large rock into small rock */
  { ITEM_HAMMER, SH_ROCK_SM,   SH_DIRT,    4 }, /* FIXME: mine gems! */

  { ITEM_ROCK,   SH_FLOWER1_A, SH_SEEDS_A /* FIXME */, 1}, /* hit flower with rock to get seeds */
  { ITEM_ROCK,   SH_FLOWER1_B, SH_SEEDS_B /* FIXME */, 1}, /* hit flower with rock to get seeds */
  { ITEM_ROCK,   SH_FLOWER2_A, SH_SEEDS_A /* FIXME */, 1}, /* hit flower with rock to get seeds */
  { ITEM_ROCK,   SH_FLOWER2_B, SH_SEEDS_B /* FIXME */, 1}, /* hit flower with rock to get seeds */

  { ITEM_NONE, SH_DIRT, SH_DIRT }, /* marks end of the list */
};

/* For each pair of:
 *   0. the item from inventory, being combined with...  (ITEM_...)
 *   1. ...the shape on the map                          (SH...)
 * we have:
 *   2. what item you create & collect                   (ITEM_...)
 *   3. how much effort it takes (tired increase amount)
 */
unsigned char tools_onhand[][4] = {
  { ITEM_HAMMER, SH_SEEDS_A,   ITEM_FLOUR, 1 }, /* seeds turn to flour */
  { ITEM_HAMMER, SH_SEEDS_B,   ITEM_FLOUR, 1 }, /* seeds turn to flour */

  { ITEM_NONE,   SH_DIRT,      ITEM_NONE, 0}, /* marks end of the list */
};

int use_tool(int item, int wx, int wy) {
  int found, i, free_slot;
  int wyy;

  wyy = wy * W;

  /* Try to use a tool to change something on the map */
  found = -1;
  for (i = 0; tools_onland[i][0] != ITEM_NONE && found == -1; i++) {
    if (tools_onland[i][0] == items[item] & tools_onland[i][1] == map[wyy + wx]) {
      found = i;
    }
  }

  if (found != -1) {
    map[wyy + wx] = tools_onland[found][2];

    tired = tired + tools_onland[found][3];
    return(USE_TOOL_NONE);
  }


  /* No luck? Try to use a tool to create an item */
  for (i = 0; tools_onhand[i][0] != ITEM_NONE && found == -1; i++) {
    if (tools_onhand[i][0] == items[item] & tools_onhand[i][1] == map[wyy + wx]) {
      found = i;
    }
  }

  if (found == -1) {
    return(USE_TOOL_BAD);
  }

  free_slot = get_free_item_slot();
  if (free_slot == MAX_ITEMS) {
    return(USE_TOOL_FULL);
  }

  items[free_slot] = tools_onhand[found][2];
  map[wyy + wx] = SH_DIRT;
  tired = tired + tools_onhand[found][3];

  return(items[free_slot]);
}
