/*
  ppm-to-tilefnt.c

  For C8zy.
  Bill Kendrick <bill@newbreedsoftware.com>

  A tool to convert a 256x64 (32x4) tile image (raw binary PPM P6 format)
  into a pair character sets (fonts), representing the top and
  bottom halves of the tiles.  Each 2x2 set of tiles (8x16 pixels)
  only need 2x1 of character graphics, thanks to a Display List
  containing LMS (load memory scan) instructions that repeat every
  other row of screen memory, and a DLI (Display List Interrupt)
  routine that toggles between the two fonts every other line.

  (Originally from "rpg" graphics demo, 2023.)

  2025-03-09 - 2025-03-09
*/

#include <stdio.h>
#include <stdlib.h>

/* Colors we expect */
int rgb[5][3] = {
  {0xf7, 0xe8, 0x86}, // 0: yellow (sand) [background]
  {0x00, 0x00, 0x00}, // 1: black
  {0x6b, 0xd1, 0x8e}, // 2: green
  {0xa7, 0x50, 0x51}, // 3: red
  {0x52, 0x77, 0xce}, // also 3: blue (alt. color when high bit set)
};

unsigned char shift[4] = { 6, 4, 2, 0 };

unsigned char bits(unsigned char byte);

int main(void) {
  int xx, yy, x, y, i, segment, chr, row;
  unsigned char bitmap[16 * 4][4 * 32];
  unsigned char byte, c;
  char str[1024];

  fgets(str, sizeof(str), stdin);
  // fprintf(stderr, "%s\n", str);
  fgets(str, sizeof(str), stdin);
  // fprintf(stderr, "%s\n", str);
  fgets(str, sizeof(str), stdin);
  // fprintf(stderr, "%s\n", str);
  fgets(str, sizeof(str), stdin);
  // fprintf(stderr, "%s\n", str);

  for (yy = 0; yy < 16 * 4; yy++) {
    for (xx = 0; xx < 4 * 32 * 2; xx += 2) {
      c = fgetc(stdin);
      // fprintf(stderr, "%02x", c);
      bitmap[yy][xx / 2] = c;
      for (i = 0; i < 5; i++) {
        c = fgetc(stdin);
      }
    }
  }

  for (segment = 0; segment < 2; segment++) {
    for (row = 0; row < 4; row++) {
      for (x = 0; x < 32; x++) {
        xx = x * 4;
        for (y = 0; y < 8; y++) {
          yy = (row * 16) + (segment * 8) + y;
          // fprintf(stderr, "seg=%d row=%d x=%03d y=%03d (%03d,%03d)", segment, row, x, y, xx, yy);

          byte = 0;
          for (i = 0; i < 4; i++) {
            // fprintf(stderr, "%02x ", bitmap[yy][xx + i]);
            byte <<= 2;
            byte += bits(bitmap[yy][xx + i]);
          }
          fputc(byte, stdout);
          // fprintf(stderr, "\n");
        }
      }
      // fprintf(stderr, "\n");
    }
  }
}

unsigned char bits(unsigned char byte) {
  int i, fnd;

  fnd = -1;
  for (i = 0; i < 5 && fnd == -1; i++) {
    if (rgb[i][0] == byte) {
      if (i == 4)
        fnd = 3;
      else
        fnd = i;
    }
  }
  if (fnd == -1) {
    fprintf(stderr, "Cannot find color with rgb 0x%02x!\n", byte);
    exit(1);
  }
  return (fnd);
}

